from django.db import models
from datetime import datetime
from django.core.validators import MaxValueValidator, MinValueValidator
from django.core.exceptions import ValidationError

from django.urls import reverse

class Author(models.Model):
    first_name = models.CharField(max_length = 50)
    last_name = models.CharField(max_length = 50)
    age = models.IntegerField(default = 0)
    nationality = models.CharField(max_length = 50)
    bio = models.TextField(max_length = 700)

    def __str__(self):
        return self.first_name + ' ' + self.last_name
    
    def get_absolute_url(self):
        return reverse('bookshelf:author_details', kwargs={'pk': self.pk})
    
    def get_books(self):
        books_list = Books.objects.filter(author=self)
        return books_list

def verify_ISBN(value):
    if value.isdigit() == False:
        raise ValidationError('ISBN must only be integers.')
    if int(len(value)) != 13:
        raise ValidationError('ISBN must be a 13 digit number.')
    
def verify_wordCount(value):
    word_count = int(len(value.split()))
    if word_count < 100:
        raise ValidationError('Word count is less than 100. Required range is 100 to 200 words.')
    if word_count > 200:
        raise ValidationError('Word count is more than 200. Required range is 100 to 200 words.')
    
class Books(models.Model):
    title = models.CharField(max_length = 70)
    author = models.ForeignKey(Author, on_delete = models.CASCADE)
    publisher = models.CharField(max_length = 70)
    year_published = models.IntegerField(null = True, blank = True,
        validators=[MinValueValidator(1000), MaxValueValidator(datetime.now().year)]
        )
    ISBN = models.CharField(max_length = 13, validators = [verify_ISBN])
    blurb = models.TextField(validators = [verify_wordCount])
    
    def __str__(self):
        return self.title
    
    def get_author(self):
        return self.author
    
    def get_absolute_url(self):
        return reverse('bookshelf:book_details', kwargs={'pk': self.pk})